//+------------------------------------------------------------------+
//|                                             CSignalEngulfing.mqh |
//|                                                    Thomas Bassen |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Thomas Bassen"
#property link      "https://www.mql5.com"

// include CCandlePattern class
#include "acandlepatterns.mqh"
//+------------------------------------------------------------------+
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of Engulfing Pattern                                                     |
//| Type=Signal                                                      |
//| Name=EngulfingPattern                                            |
//| Class=CSignalEngulfing                                           |
//| Page=                                                            |
//| Parameter=Limit,double,0.0                                       |
//| Parameter=StopLoss,double,50.0                                   |
//| Parameter=TakeProfit,double,50.0
//| Parameter=MAPeriod,int,12                                        |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
class CSignalEngulfing  :  public CCandlePattern
   {
protected:
     //--- Setup parameters     
     double          m_stop_loss;            // level to place a stop loss order relative to the open price
     double          m_take_profit;          // level to place a take profit order relative to the open price
     double          m_limit;                // level to place a pending order

        
public:
     //---constructor
                  CSignalEngulfing();
     //--- input parameters initialization methods
     void              MAPeriod(int period) { m_ma_period=period;}            
     //--- Methods to set the parameters
     void               StopLoss(double value)              { m_stop_loss=value;   }
     void               TakeProfit(double value)            { m_take_profit=value; }
     void               Limit(double value)                 { m_limit=value;       }
     void               Expiration(int value)               { m_expiration=value;  }
     
     //--- Methods to validate the parameters
     virtual bool       ValidationSettings();
     
     //--- checking of trade signals
     virtual bool      CheckOpenLong(double &price,double &sl,double &tp,datetime &expiration);
     virtual bool      CheckCloseLong(double &price);
     virtual bool      CheckOpenShort(double &price,double &sl,double &tp,datetime &expiration);
     virtual bool      CheckCloseShort(double &price);
     
     //--- method for checking of a certiain candlestick pattern
     bool              CheckCandlestickPattern(ENUM_CANDLE_PATTERNS CandlePattern);
     //--- methods for checking of bullish/bearish candlestick pattern
     bool              CheckPatternAllBullish();
     bool              CheckPatternAllBearish();
     //--- methods for checking of a certain candlestick model
     bool              CheckPatternBullishEngulfing();
     bool              CheckPatternBearishEngulfing();                 
   };
   
     bool CSignalEngulfing ::ValidationSettings()
  {
//--- call of ValidationSettings of parent CCandlePattern class
   if(!CCandlePattern::ValidationSettings()) return(false);
//--- your code

//--- ok
   return(true);
  }
  
  
//+------------------------------------------------------------------+
//| Checking condition of long position opening                      |
//+------------------------------------------------------------------+
bool CSignalEngulfing::CheckOpenLong(double &price,double &sl,double &tp,datetime &expiration)
  {
//--- check conditions to open long position
//--- check "bullish engulfing"
   if (CheckCandlestickPattern(CANDLE_PATTERN_BULLISH_ENGULFING)) return(true);
//--- no signal
   return(false);
  }
  
//-------------------------------------------------------------------+
//| Checking condition of long position closing                      |
//+------------------------------------------------------------------+
bool CSignalEngulfing::CheckCloseLong(double &price)
  {
//--- check conditions to close long position
//--- for example, let's check formation of one of the bearish pattern:
   if (CheckPatternAllBearish()) return(true);
//--- no signal
   return(false);
  }
  
//-------------------------------------------------------------------+
//| Checking condition of short position opening                     |
//+------------------------------------------------------------------+
bool CSignalEngulfing::CheckOpenShort(double &price,double &sl,double &tp,datetime &expiration)
  {
//--- check conditions to open short position
//--- for example, let's check formation of "3 black crows" pattern:
    if (CheckCandlestickPattern(CANDLE_PATTERN_BEARISH_ENGULFING)) return(true);
//--- no signal
   return(false);
  }
  
//-------------------------------------------------------------------+
//| Checking condition of short position closing                     |
//+------------------------------------------------------------------+
bool CSignalEngulfing::CheckCloseShort(double &price)
  {
//--- check conditions to close short position
//--- for example, let's check formation of one of the bullish pattern:
   if (CheckPatternAllBullish()) return(true);
//--- no signal
   return(false);
  }              
   